﻿using System.Threading;
using System.Threading.Tasks;


namespace System.Collections.Generic
{
    public interface IAsyncEnumerable<out T>
    {
        IAsyncEnumerator<T> GetAsyncEnumerator(CancellationToken cancellationToken = default);
    }

    public interface IAsyncEnumerator<out T> : IAsyncDisposable
    {
        ValueTask<bool> MoveNextAsync();
        T Current { get; }
    }
}

namespace System
{
    public interface IAsyncDisposable
    {
        ValueTask DisposeAsync();
    }
}

namespace System.Runtime.CompilerServices
{
    public struct AsyncIteratorMethodBuilder
    {
        // Fields
        private AsyncTaskMethodBuilder _methodBuilder;

        // Methods
        public void AwaitOnCompleted<TAwaiter, TStateMachine>(ref TAwaiter awaiter, ref TStateMachine stateMachine) where TAwaiter : INotifyCompletion where TStateMachine : IAsyncStateMachine
        {
            this._methodBuilder.AwaitOnCompleted<TAwaiter, TStateMachine>(ref awaiter, ref stateMachine);
        }

        public void AwaitUnsafeOnCompleted<TAwaiter, TStateMachine>(ref TAwaiter awaiter, ref TStateMachine stateMachine) where TAwaiter : ICriticalNotifyCompletion where TStateMachine : IAsyncStateMachine
        {
            this._methodBuilder.AwaitUnsafeOnCompleted<TAwaiter, TStateMachine>(ref awaiter, ref stateMachine);
        }

        public void Complete()
        {
            this._methodBuilder.SetResult();
        }

        public static AsyncIteratorMethodBuilder Create()
        {
            return new AsyncIteratorMethodBuilder();
        }

        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public void MoveNext<TStateMachine>(ref TStateMachine stateMachine) where TStateMachine : IAsyncStateMachine
        {
            // AsyncMethodBuilderCore.Start<TStateMachine>(ref stateMachine);
        }

        // Properties
        /*
        internal object ObjectIdForDebugger
        {
            get
            {
                return this._methodBuilder.ObjectIdForDebugger;
            }
        }
        */
    }
}

namespace System.Threading.Tasks.Sources
{
    public struct ManualResetValueTaskSourceCore<TResult>
    {
        private TResult _result;

        private object _dummy;

        private int _dummyPrimitive;

        public bool RunContinuationsAsynchronously
        {
            get
            {
                throw null;
            }
            set
            {
            }
        }

        public short Version
        {
            get
            {
                throw null;
            }
        }

        public TResult GetResult(short token)
        {
            throw null;
        }

        public ValueTaskSourceStatus GetStatus(short token)
        {
            throw null;
        }

        public void OnCompleted(Action<object> continuation, object state, short token, ValueTaskSourceOnCompletedFlags flags)
        {
        }

        public void Reset()
        {
        }

        public void SetException(Exception error)
        {
        }

        public void SetResult(TResult result)
        {
        }
    }
}
